# (c) 2023 Christophe Gueneau
from time import *

class Rtc_grove():
    """Classe définissant l'horlogue temps réel RTC I2C Grove"""
    def __init__(self, i2c):
        self._i2c = i2c
        self._adr_i2c = 0x68                     # adresse I2C par défaut
      
    def _dec_bcd(self, val):
        return (val // 10) << 4 | (val % 10)

    def _bcd_dec(self, val):
        return ((val >> 4) * 10) + (val & 0x0F)
      
    def lectureRTC(self):
        donnee = self._i2c.readfrom_mem(self._adr_i2c, 0, 7)
        return (
            self._bcd_dec(donnee[6]) + 2000,                 # année
            self._bcd_dec(donnee[5]),                        # mois
            self._bcd_dec(donnee[4]),                        # date 1-31
            self._bcd_dec(donnee[3]),                        # jour de la semaine 1-7
            self._bcd_dec(donnee[2]),                        # heure
            self._bcd_dec(donnee[1]),                        # minute
            self._bcd_dec(donnee[0] & 0b0111_1111)           # seconde, msb = CH, 1=arret, 0=marche
        )
    
    def ecritureRTC(self, dateHeure):
        lst_dateHeure=[0,0,0,0,0,0,0]
        lst_dateHeure[6] = self._dec_bcd(dateHeure[0] - 2000)                # année
        lst_dateHeure[5] = self._dec_bcd(dateHeure[1])                       # mois
        lst_dateHeure[4] = self._dec_bcd(dateHeure[2])                       # date 1-31
        lst_dateHeure[3] = self._dec_bcd(dateHeure[3])                       # jour de la semaine 1-7
        lst_dateHeure[2] = self._dec_bcd(dateHeure[4])                       # heure
        lst_dateHeure[1] = self._dec_bcd(dateHeure[5])                       # minute
        lst_dateHeure[0] = self._dec_bcd(dateHeure[6]) & 0b0111_1111       # seconde, msb = CH, 1=arret, 0=marche
        donnee= bytearray(lst_dateHeure)
        self._i2c.writeto_mem(self._adr_i2c, 0, donnee)
            
    def marcheRTC(self):
        seconde = self._i2c.readfrom_mem(self._adr_i2c, 0, 7)[0] & 0b0111_1111   # msb = CH, 0=marche
        donnee=bytearray([seconde])
        self._i2c.writeto_mem(self._adr_i2c, 0x00, donnee)        
    
    def arretRTC(self):
        seconde = self._i2c.readfrom_mem(self._adr_i2c, 0, 7)[0] | 0b1000_0000   # msb = CH, 1=arret
        donnee=bytearray([seconde])
        self._i2c.writeto_mem(self._adr_i2c, 0x00, donnee)          
        
        
if __name__ == '__main__':
    
    from machine import Pin, I2C
    
    # Crée un objet I2C
    i2c0 = I2C(0, scl=Pin(9), sda=Pin(8), freq=400000)    # I2C0 utilisé  

    # Crée un objet Rtc_grove
    rtc = Rtc_grove(i2c0)

    # écrit une date spécifique
    rtc.ecritureRTC([2022, 10, 31, 1, 17, 24, 0])     # 2022/10/31 17:24:00  1=lundi

    # lecture de la date et de l'heure courante
    print(rtc.lectureRTC())

    # mise à l'arrêt horloge
    rtc.arretRTC()      

    # reprise horloge
    rtc.marcheRTC()     

    
    
    